<?php

/**
 * The RedirectGuard prevents unsafe redirects from proceeding.
 * Partially backported to WSC 5.3.
 *
 * Current checks:
 * - Redirects to non-standard ports.
 * - Redirects non non-HTTP schemes.
 *
 * @author  SoftCreatR Media, Tim Duesterhus
 * @copyright   2001-2021 WoltLab GmbH
 * @license GNU Lesser General Public License <http://opensource.org/licenses/lgpl-license.php>
 * @package WoltLabSuite\Core\System\Io\Http
 * @since   5.4
 */

namespace wcf\system\io\http;

use GuzzleHttp\Exception\BadResponseException;
use Psr\Http\Message\{RequestInterface, ResponseInterface, UriInterface};

final class ScRedirectGuard
{
    /**
     * @var null|callable
     */
    private $next;

    /**
     * @param callable|null $next
     */
    public function __construct($next = null)
    {
        $this->next = $next;
    }

    public function __invoke(RequestInterface $request, ResponseInterface $response, UriInterface $uri)
    {
        if (null !== $uri->getPort()) {
            throw new BadResponseException(
                "Refusing to follow redirects to non-standard ports.",
                $request,
                $response
            );
        }

        if ('' !== $uri->getScheme() && 0 !== strncmp($uri->getScheme(), 'http', 4)) {
            throw new BadResponseException(
                "Refusing to follow redirects to non-HTTP schemes.",
                $request,
                $response
            );
        }

        if (($next = $this->next)) {
            return $next($request, $response, $uri);
        }

        return null;
    }
}
